/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef PERCEPTION_IMPL_TYPE_BBOX2D_H
#define PERCEPTION_IMPL_TYPE_BBOX2D_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"

namespace perception {
struct BBox2D {
    ::Double xmin{ 0.0 };
    ::Double ymin{ 0.0 };
    ::Double xmax{ 0.0 };
    ::Double ymax{ 0.0 };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(xmin);
        fun(ymin);
        fun(xmax);
        fun(ymax);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(xmin);
        fun(ymin);
        fun(xmax);
        fun(ymax);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("xmin", xmin);
        fun("ymin", ymin);
        fun("xmax", xmax);
        fun("ymax", ymax);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("xmin", xmin);
        fun("ymin", ymin);
        fun("xmax", xmax);
        fun("ymax", ymax);
    }

    friend bool operator==(const ::perception::BBox2D& lhs, const ::perception::BBox2D& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.xmin - rhs.xmin)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ymin - rhs.ymin)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.xmax - rhs.xmax)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ymax - rhs.ymax)) < DBL_EPSILON);
    }
};
} // namespace perception


#endif // PERCEPTION_IMPL_TYPE_BBOX2D_H
